"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.retry = void 0;
const logger = require("../../log");
const retry = async (fn, { retries, operation } = { retries: 30, operation: '' }, attempt = 0) => {
    try {
        return await fn();
    }
    catch (error) {
        logger.debug({ attempt, operation, error }, 'failed to execute retry');
        if (retries <= 0) {
            throw error;
        }
        const backoff = 2 ** attempt * 100;
        const timeout = backoff > 60000 ? 60000 : backoff;
        logger.warn({ attempt, operation, timeout }, `waiting for ${timeout}ms before next try`);
        await sleep(timeout);
        attempt++;
        return (0, exports.retry)(fn, { retries: retries - 1, operation }, attempt);
    }
};
exports.retry = retry;
const sleep = (ms = 0) => new Promise((resolve) => setTimeout(resolve, ms));
//# sourceMappingURL=exponential-backoff.js.map