var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _a, _b;
var _DispatcherClient_instances, _DispatcherClient_url, _DispatcherClient_hostname, _DispatcherClient_id, _DispatcherClient_version, _DispatcherClient_makeRequest;
const { maskToken } = require('./token');
const logger = require('./log');
const config = require('./config');
const crypto = require('crypto');
const { axiosInstance } = require('./axios');
const { v4: uuid } = require('uuid');
class DispatcherClient {
    constructor(url, hostname, id, version) {
        _DispatcherClient_instances.add(this);
        _DispatcherClient_url.set(this, void 0);
        _DispatcherClient_hostname.set(this, void 0);
        _DispatcherClient_id.set(this, void 0);
        _DispatcherClient_version.set(this, void 0);
        __classPrivateFieldSet(this, _DispatcherClient_url, url, "f");
        __classPrivateFieldSet(this, _DispatcherClient_hostname, hostname, "f");
        __classPrivateFieldSet(this, _DispatcherClient_id, id || 0, "f");
        __classPrivateFieldSet(this, _DispatcherClient_version, version || '2022-12-02~experimental', "f");
    }
    async serverStarting() {
        await __classPrivateFieldGet(this, _DispatcherClient_instances, "m", _DispatcherClient_makeRequest).call(this, { requestType: 'server-starting' }, `${__classPrivateFieldGet(this, _DispatcherClient_url, "f")}/internal/brokerservers/${__classPrivateFieldGet(this, _DispatcherClient_id, "f")}`, 'post', {
            data: {
                attributes: {
                    health_check_link: `http://${__classPrivateFieldGet(this, _DispatcherClient_hostname, "f")}/healthcheck`,
                },
            },
        });
    }
    async serverStopping(cb) {
        await __classPrivateFieldGet(this, _DispatcherClient_instances, "m", _DispatcherClient_makeRequest).call(this, { requestType: 'server-stopping' }, `${__classPrivateFieldGet(this, _DispatcherClient_url, "f")}/internal/brokerservers/${__classPrivateFieldGet(this, _DispatcherClient_id, "f")}`, 'delete', cb);
    }
    async clientConnected(token, clientId) {
        const maskedToken = maskToken(token);
        await __classPrivateFieldGet(this, _DispatcherClient_instances, "m", _DispatcherClient_makeRequest).call(this, { maskedToken, clientId, requestType: 'client-connected' }, `${__classPrivateFieldGet(this, _DispatcherClient_url, "f")}/internal/brokerservers/${__classPrivateFieldGet(this, _DispatcherClient_id, "f")}/connections/${hash(token)}${clientId ? `?broker_client_id=${clientId}` : ''}`, 'post', {
            data: {
                attributes: {
                    health_check_link: `http://${__classPrivateFieldGet(this, _DispatcherClient_hostname, "f")}/healthcheck`,
                },
            },
        });
    }
    async clientDisconnected(token, clientId) {
        const maskedToken = maskToken(token);
        await __classPrivateFieldGet(this, _DispatcherClient_instances, "m", _DispatcherClient_makeRequest).call(this, { maskedToken, clientId, requestType: 'client-disconnected' }, `${__classPrivateFieldGet(this, _DispatcherClient_url, "f")}/internal/brokerservers/${__classPrivateFieldGet(this, _DispatcherClient_id, "f")}/connections/${hash(token)}${clientId ? `?broker_client_id=${clientId}` : ''}`, 'delete');
    }
}
_DispatcherClient_url = new WeakMap(), _DispatcherClient_hostname = new WeakMap(), _DispatcherClient_id = new WeakMap(), _DispatcherClient_version = new WeakMap(), _DispatcherClient_instances = new WeakSet(), _DispatcherClient_makeRequest = async function _DispatcherClient_makeRequest(logContext, url, method, requestBody, cb) {
    const requestId = uuid();
    // version *must* be provided
    const urlWithVersion = new URL(url);
    urlWithVersion.searchParams.append('version', __classPrivateFieldGet(this, _DispatcherClient_version, "f"));
    url = urlWithVersion.toString();
    try {
        const response = await axiosInstance.request({
            url,
            method,
            data: requestBody && JSON.stringify(requestBody),
            headers: {
                'Content-Type': 'application/vnd.api+json',
                Connection: 'Keep-Alive',
                'Keep-Alive': 'timeout=60, max=10',
                'Snyk-Request-Id': requestId,
            },
        });
        const statusCode = response.status;
        const headers = response.headers;
        const body = response.data;
        if (statusCode >= 300) {
            logger.error({
                ...logContext,
                requestId,
                statusCode,
                headers,
                body,
                dispatcherUrl: __classPrivateFieldGet(this, _DispatcherClient_url, "f"),
                dispatcherVersion: __classPrivateFieldGet(this, _DispatcherClient_version, "f"),
                serverId: __classPrivateFieldGet(this, _DispatcherClient_id, "f"),
            }, 'received unexpected status code communicating with Dispatcher');
        }
        else {
            logger.trace({ ...logContext, serverId: __classPrivateFieldGet(this, _DispatcherClient_id, "f"), requestId }, 'successfully sent request to Dispatcher');
        }
    }
    catch (e) {
        logger.error({
            ...logContext,
            requestId,
            errorMessage: e.message,
            stackTrace: new Error('stack generator').stack,
            dispatcherUrl: __classPrivateFieldGet(this, _DispatcherClient_url, "f"),
            dispatcherVersion: __classPrivateFieldGet(this, _DispatcherClient_version, "f"),
            serverId: __classPrivateFieldGet(this, _DispatcherClient_id, "f"),
        }, 'received error communicating with Dispatcher');
    }
    if (cb)
        cb();
};
function hash(token) {
    const shasum = crypto.createHash('sha256');
    shasum.update(token);
    return shasum.digest('hex');
}
let clientConnected;
let clientDisconnected;
let serverStarting;
let serverStopping;
if (config.dispatcherUrl) {
    const kc = new DispatcherClient(config.dispatcherUrl, config.hostname, (_a = config.hostname) === null || _a === void 0 ? void 0 : _a.substring(((_b = config.hostname) === null || _b === void 0 ? void 0 : _b.lastIndexOf('-')) + 1), config.dispatcherVersion);
    clientConnected = async function (token, clientId) {
        return kc.clientConnected(token, clientId);
    };
    clientDisconnected = async function (token, clientId) {
        return kc.clientDisconnected(token, clientId);
    };
    serverStarting = async function () {
        return kc.serverStarting();
    };
    serverStopping = async function (cb) {
        return kc.serverStopping(cb);
    };
}
else {
    logger.error('DISPATCHER_URL not set - creating no-op functions to ensure Server still functions');
    clientConnected = async function () {
        logger.trace('client connected - no-op instead of notifying dispatcher');
    };
    clientDisconnected = async function () {
        logger.trace('client disconnected - no-op instead of notifying dispatcher');
    };
    serverStarting = async function () {
        logger.info('server started - no-op instead of notifying dispatcher');
    };
    serverStopping = async function (cb) {
        logger.info('server stopping - no-op instead of notifying dispatcher');
        cb();
    };
}
module.exports = {
    clientConnected,
    clientDisconnected,
    serverStarting,
    serverStopping,
    hash,
};
//# sourceMappingURL=dispatcher.js.map